import glob
import json
import os
import pathlib
import plistlib
import sqlite3
import textwrap

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from tools.ccl import ccl_bplist
from helpers import is_platform_windows, kmlgen, open_sqlite_db_readonly, timeline, tsv
from tools.parse3 import ParseProto

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class KnowledgeC(ab.Artifact):

    _name = 'KnowledgeC'
    _search_dirs = '**/CoreDuet/Knowledge/knowledgeC.db'
    _category = 'KnowledgeC'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("12"):
            logfunc(
                "Unsupported version for KnowledgC Inferred Motion on iOS " + iOSversion
            )
        else:
            file_found = str(files_found[0])
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zsecondsfromgmt/3600,
            zobject.zvalueinteger,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname = '/inferred/motion'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Inferred Motion')
                report.start_artifact_report(
                    report_folder, 'Inferred Motion', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'GMT Offset',
                    'Value',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Inferred Motion'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Inferred Motion'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Inferred Motion')

        cursor = db.cursor()
        cursor.execute(
            """
            select
            datetime(zobject.zcreationdate+978307200,'unixepoch'),
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__title,
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate+978307200,'unixepoch'),
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_date+978307200,'unixepoch'),
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_enddate+978307200,'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            order by 'entry creation'
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        for row in all_rows:
            data_list.append(
                (
                    row[0],
                    row[1],
                    row[2],
                    row[3],
                    row[4],
                    row[5],
                    row[6],
                    row[7],
                    row[8],
                    row[9],
                    row[10],
                )
            )

        description = ''
        report = ArtifactHtmlReport('KnowledgeC App Activity')
        report.start_artifact_report(report_folder, 'App Activity', description)
        report.add_script()
        data_headers = (
            'Entry Creation',
            'Day of the Week',
            'Start',
            'End',
            'Value String',
            'Activity Type',
            'Title',
            'Expiration Date',
            'Content URL',
            'Calendar Date',
            'Calendar End Date',
        )
        report.write_artifact_data_table(data_headers, data_list, file_found)
        report.end_artifact_report()

        tsvname = 'KnowledgeC App Activity'
        tsv(report_folder, data_headers, data_list, tsvname)

        tlactivity = 'KnowledgeC App Activity'
        timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zsource.zgroupid,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__contentdescription,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__suggestedinvocationphrase,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelateduniqueidentifier,
            zsource.zsourceid,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityuuid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__title,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []

                if version.parse(iOSversion) >= version.parse("12"):
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                                row[8],
                                row[9],
                                row[10],
                                row[11],
                                row[12],
                                row[13],
                                row[14],
                                row[15],
                                row[16],
                            )
                        )

                    report = ArtifactHtmlReport('KnowledgeC Application Activity')
                    report.start_artifact_report(report_folder, 'Application Activity')
                    report.add_script()
                    data_headers = (
                        'Start',
                        'End',
                        'Bundle ID',
                        'Group ID',
                        'Activity Type',
                        'Content Description',
                        'User Activity Required String',
                        'Content URL',
                        'Suggest Invocation Phrase',
                        'Unique ID',
                        'Source ID',
                        'ID',
                        'Activity UUID',
                        'Day of Week',
                        'GMT Offset',
                        'Entry Creation',
                        'Expiration Date',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'KnowledgeC Application Activity'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'KnowledgeC Application Activity'
                    timeline(report_folder, tlactivity, data_list, data_headers)
                else:
                    for row in all_rows:
                        data_list.append(
                            (
                                row[0],
                                row[1],
                                row[2],
                                row[3],
                                row[4],
                                row[5],
                                row[6],
                                row[7],
                                row[8],
                                row[9],
                            )
                        )
                    report = ArtifactHtmlReport('KnowledgeC Application Activity')
                    report.start_artifact_report(report_folder, 'Application Activity')
                    report.add_script()
                    data_headers = (
                        'Start',
                        'End',
                        'Bundle ID',
                        'Activity Type',
                        'Title',
                        'Content URL',
                        'Day of Week',
                        'GMT Offset',
                        'Entry Creation',
                        'Expiration Date',
                    )
                    report.write_artifact_data_table(
                        data_headers, data_list, file_found
                    )
                    report.end_artifact_report()

                    tsvname = 'KnowledgeC Application Activity'
                    tsv(report_folder, data_headers, data_list, tsvname)

                    tlactivity = 'KnowledgeC Application Activity'
                    timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Application Activity')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring ,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__title,
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_date + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_enddate + 978307200, 'unixepoch'),
            zsource.zsourceid,
            case zobject.zstartdayofweek
            when "1" then "sunday"
            when "2" then "monday"
            when "3" then "tuesday"
            when "4" then "wednesday"
            when "5" then "thursday"
            when "6" then "friday"
            when "7" then "saturday"
            end ,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from
            zobject
            left join
            zstructuredmetadata
            on zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            left join
            zsource
            on zobject.zsource = zsource.z_pk
            where
            zstreamname = "/app/activity"
            and (zvaluestring = "com.apple.mobilecal" or zvaluestring = "com.apple.ical")
            """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring ,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__title,
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_date + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.zcom_apple_calendaruikit_useractivity_enddate + 978307200, 'unixepoch'),
            zsource.zsourceid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end ,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            and (zvaluestring = 'com.apple.mobilecal' or zvaluestring = 'com.apple.ical')
                        """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("12"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                        )
                    )

                report = ArtifactHtmlReport(
                    'InteractionC Application Activity Calendar'
                )
                report.start_artifact_report(
                    report_folder, 'Application Activity Calendar'
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Activity Type',
                    'User Activity Required String',
                    'Title',
                    'Calendar Date',
                    'Calendar End Date',
                    'Source ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                    'Expiration Date',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application Activity Calendar'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application Activity Calendar'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                        )
                    )

                report = ArtifactHtmlReport('InteractionC Application Activty Calendar')
                report.start_artifact_report(
                    report_folder, 'Application Activity Calendar'
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Activity Type',
                    'Title',
                    'Expiration Date',
                    'Calendar Date',
                    'Calendar End Date',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application Activity Calendar'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application Activity Calendar'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Application Activity Calendar')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__contentdescription,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelateduniqueidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityuuid,
            zsource.zsourceid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            and (zobject.zvaluestring = 'com.apple.mobilesafari' or zobject.zvaluestring = 'com.apple.safari')
                """
            )
        else:
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelateduniqueidentifier,
            zsource.zsourceid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/activity'
            and (zobject.zvaluestring = 'com.apple.mobilesafari' or zobject.zvaluestring = 'com.apple.safari')
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []

            if version.parse(iOSversion) >= version.parse("12"):

                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                        )
                    )

                report = ArtifactHtmlReport('KnowledgeC Application Activity Safari')
                report.start_artifact_report(
                    report_folder, 'Application Activity Safari'
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Activity Type',
                    'Content Description',
                    'Content URL',
                    'User Activity Required String',
                    'ID',
                    'Unique ID',
                    'Activity UUID',
                    'Source ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                    'Expiration Date',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application Activity Safari'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application Activity Safari'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('KnowledgeC Application Activty Safari')
                report.start_artifact_report(
                    report_folder, 'Application Activity Safari'
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Activity Type',
                    'Content URL',
                    'ID',
                    'Unique ID',
                    'Source ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                    'Expiration Date',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application Activity Safari'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application Activity Safari'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Appplication Activity Safari')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from zobject
            where zstreamname = '/app/relevantShortcuts'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Application Relevant Shortcuts')
                report.start_artifact_report(
                    report_folder, 'App Relevant Shortcuts', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Day of the Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application Relevant Shortcuts'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application Relevant Shortcuts'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Relevant Shortcuts')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zvalueinteger
            when '0' then 'no'
            when '1' then 'yes'
            end,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from zobject
            where
            zobject.zstreamname = '/display/isBacklit'
            """
            )
        elif version.parse(iOSversion) == version.parse("11"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zvalueinteger
            when '0' then 'no'
            when '1' then 'yes'
            end,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from zobject
            where
            zobject.zstreamname = '/display/isBacklit'
            """
            )
        else:
            logfunc("Unsupported version for KnowledgC Backlit" + iOSversion)
            return ()

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("12"):
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('KnowledgeC Device is Backlit')
                report.start_artifact_report(report_folder, 'Device is Backlit')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Screen is Backlit',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Device is Backlit'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Device is Backlit'
                timeline(report_folder, tlactivity, data_list, data_headers)

            elif version.parse(iOSversion) == version.parse("11"):
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                    )

                report = ArtifactHtmlReport('KnowledgeC Device is Backlit')
                report.start_artifact_report(report_folder, 'Device is Backlit')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Screen is Backlit',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Device is Backlit'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Device is Backlit'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Device is Backlit')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluedouble,
            (zobject.zenddate - zobject.zstartdate),
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname like '/device/BatteryPercentage'
                """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            for row in all_rows:
                data_list.append(
                    (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                )

            description = ''
            report = ArtifactHtmlReport('KnowledgeC Battery Level')
            report.start_artifact_report(report_folder, 'Battery Level', description)
            report.add_script()
            data_headers = (
                'Start',
                'End',
                'Battery Level',
                'Usage in Seconds',
                'Day of the Week',
                'GMT Offset',
                'Entry Creation',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'KnowledgeC Battery Level'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'KnowledgeC Battery Level'
            timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zstructuredmetadata.z_dkbluetoothmetadatakey__address,
            zstructuredmetadata.z_dkbluetoothmetadatakey__name,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject,
            zstructuredmetadata
            on zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            where
            zstreamname = '/bluetooth/isConnected'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Bluetooth Connections')
                report.start_artifact_report(
                    report_folder, 'Bluetooth Connections', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bluetooth Address',
                    'Bluetooth Name',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Bluetooth Connections'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Bluetooth Connections'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Bluetooth Connections')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zvalueinteger
            when '0' then 'disconnected'
            when '1' then 'connected'
            end,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname is '/carplay/isConnected'
                """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Car Play Connections')
                report.start_artifact_report(
                    report_folder, 'Car Play Connections', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Car Play Connected',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Car Play Connections'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Car Play Connections'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Car Play Connections')

        if version.parse(iOSversion) >= version.parse("13"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zsource.zbundleid,
            zobject.zvaluestring,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zsource
            where zobject.zsource = zsource.z_pk
            and zstreamname = '/disk/subsystemAccess'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Disk Subsystem Access')
                report.start_artifact_report(
                    report_folder, 'Disk Subsystem Access', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Value String',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Disk Subsystem Access'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Disk Subsystem Access'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Disk Subsystem Access')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvalueinteger,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname = '/inferred/motion'
                """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Do Not Disturb')
                report.start_artifact_report(
                    report_folder, 'Do Not Disturb', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Value',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Do Not Disturb'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Do Not Disturb'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Do Not Disturb')

        data_list = []
        if version.parse(iOSversion) >= version.parse("11"):

            extension = ".bplist"
            dump = True
            # create directories
            outpath = report_folder

            try:
                os.mkdir(os.path.join(report_folder, "clean"))
                os.mkdir(os.path.join(report_folder, "dirty"))
            except OSError:
                logfunc("Error making directories")
            file_found = str(files_found[0])
            # connect sqlite databases
            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()

            # variable initializations
            dirtcount = 0
            cleancount = 0
            intentc = {}
            intentv = {}

            cursor.execute(
                """
            SELECT
            Z_PK,
            Z_DKINTENTMETADATAKEY__SERIALIZEDINTERACTION,
            Z_DKINTENTMETADATAKEY__INTENTCLASS,
            Z_DKINTENTMETADATAKEY__INTENTVERB
            FROM ZSTRUCTUREDMETADATA
            WHERE Z_DKINTENTMETADATAKEY__SERIALIZEDINTERACTION is not null
            """
            )

            all_rows = cursor.fetchall()

            for row in all_rows:
                pkv = str(row[0])
                pkvplist = pkv + extension
                f = row[1]
                intentclass = str(row[2])
                intententverb = str(row[3])
                output_file = open(
                    os.path.join(outpath, "dirty", "D_Z_PK" + pkvplist), "wb"
                )  # export dirty from DB
                output_file.write(f)
                output_file.close()

                g = open(os.path.join(outpath, "dirty", "D_Z_PK" + pkvplist), "rb")
                plistg = ccl_bplist.load(g)

                if version.parse(iOSversion) < version.parse("12"):
                    ns_keyed_archiver_objg = ccl_bplist.deserialise_NsKeyedArchiver(
                        plistg
                    )
                    newbytearray = ns_keyed_archiver_objg
                else:
                    ns_keyed_archiver_objg = ccl_bplist.deserialise_NsKeyedArchiver(
                        plistg
                    )
                    newbytearray = ns_keyed_archiver_objg["NS.data"]

                dirtcount = dirtcount + 1

                binfile = open(
                    os.path.join(outpath, "clean", "C_Z_PK" + pkvplist), "wb"
                )
                binfile.write(newbytearray)
                binfile.close()

                # add to dictionaries
                intentc["C_Z_PK" + pkvplist] = intentclass
                intentv["C_Z_PK" + pkvplist] = intententverb

                cleancount = cleancount + 1
            """
            h = open(outpath + "/StrucMetadata.html", "w")
            h.write("<html><body>")
            h.write(
                "<h2>iOS "
                + iOSversion
                + " - KnowledgeC ZSTRUCTUREDMETADATA bplist report</h2>"
            )
            h.write(
                "<style> table, td {border: 1px solid black; border-collapse: collapse;}tr:nth-child(even) {background-color: # f2f2f2;} .table th { background: # 888888; color: # ffffff}.table.sticky th{ position:sticky; top: 0; }</style>"
            )
            h.write("<br/>")
            """
            for filename in glob.glob(outpath + "/clean/*" + extension):
                p = open(filename, "rb")
                cfilename = os.path.basename(filename)
                plist = ccl_bplist.load(p)
                ns_keyed_archiver_obj = ccl_bplist.deserialise_NsKeyedArchiver(
                    plist, parse_whole_structure=True
                )  # deserialize clean
                # Get dictionary values
                A = intentc.get(cfilename)
                B = intentv.get(cfilename)

                if A is None:
                    A = "No value"
                if B is None:
                    A = "No value"

                # logfunc some values from clean bplist
                if version.parse(iOSversion) >= version.parse("13"):
                    try:
                        NSdata = ns_keyed_archiver_obj["root"]["intent"][
                            "backingStore"
                        ]["bytes"]
                    except:
                        NSdata = ns_keyed_archiver_obj["root"]["intent"][
                            "backingStore"
                        ]["data"]["NS.data"]
                        pass
                else:
                    NSdata = ns_keyed_archiver_obj["root"]["intent"]["backingStore"][
                        "data"
                    ]["NS.data"]
                    # logfunc(str(NSdata))

                parsedNSData = ""
                # Default true
                if dump == True:
                    nsdata_file = os.path.join(
                        outpath, "clean", cfilename + "_nsdata.bin"
                    )
                    binfile = open(nsdata_file, "wb")
                    if version.parse(iOSversion) >= version.parse("13"):
                        try:
                            binfile.write(
                                ns_keyed_archiver_obj["root"]["intent"]["backingStore"][
                                    "bytes"
                                ]
                            )
                        except:
                            binfile.write(
                                ns_keyed_archiver_obj["root"]["intent"]["backingStore"][
                                    "data"
                                ]["NS.data"]
                            )
                            pass
                    else:
                        binfile.write(
                            ns_keyed_archiver_obj["root"]["intent"]["backingStore"][
                                "data"
                            ]["NS.data"]
                        )
                    binfile.close()
                    messages = ParseProto(nsdata_file)
                    messages_json_dump = json.dumps(
                        messages, indent=4, sort_keys=True, ensure_ascii=False
                    )
                    parsedNSData = str(messages_json_dump).encode(
                        encoding="UTF-8", errors="ignore"
                    )

                NSstartDate = ccl_bplist.convert_NSDate(
                    (ns_keyed_archiver_obj["root"]["dateInterval"]["NS.startDate"])
                )
                NSendDate = ccl_bplist.convert_NSDate(
                    (ns_keyed_archiver_obj["root"]["dateInterval"]["NS.endDate"])
                )
                NSduration = ns_keyed_archiver_obj["root"]["dateInterval"][
                    "NS.duration"
                ]
                Siri = ns_keyed_archiver_obj["root"]["_donatedBySiri"]

                if parsedNSData:
                    parsedf = str(parsedNSData).replace("\\n", "<br>")
                else:
                    parsedf = str(NSdata).replace("\\n", "<br>")

                data_list.append(
                    (
                        str(NSstartDate),
                        str(A),
                        str(B),
                        str(Siri),
                        str(NSendDate),
                        str(NSduration),
                        parsedf,
                        (textwrap.fill(str(NSdata), width=50)),
                        cfilename,
                    )
                )

            logfunc("iOS - KnowledgeC ZSTRUCTUREDMETADATA bplist extractor")
            logfunc("By: @phillmoore & @AlexisBrignoni")
            logfunc("thinkdfir.com & abrignoni.com")
            logfunc("")
            logfunc(
                "Bplists from the Z_DKINTENTMETADATAKEY__SERIALIZEDINTERACTION field."
            )
            logfunc("Exported bplists (dirty): " + str(dirtcount))
            logfunc("Exported bplists (clean): " + str(cleancount))
            logfunc("")
            logfunc("Incepted bplist extractions in KnowledgeC.db completed")

            description = ''
            report = ArtifactHtmlReport('KnowledgeC Intents')
            report.start_artifact_report(report_folder, 'Intents', description)
            report.add_script()
            data_headers = (
                'NS Start Date',
                'Intent Class',
                'Intent Verb',
                'Siri?',
                'NS Send Date',
                'NS Duration',
                'NS Data Protobuf',
                'NS Data',
                'Traceback',
            )
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tlactivity = 'KnowledgeC Intents'
            timeline(report_folder, tlactivity, data_list, data_headers)

        if version.parse(iOSversion) >= version.parse("9"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            (zobject.zenddate-zobject.zstartdate),
            (zobject.zenddate-zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from zobject
            where zstreamname is '/app/inFocus'
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("9"):
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('KnowledgeC Application In Focus')
                report.start_artifact_report(report_folder, 'App In Focus')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Application in Focus'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Application in Focus'
                timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available in Application in Focus')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata .z_dkappinstallmetadatakey__primarycategory,
            zstructuredmetadata .z_dkappinstallmetadatakey__title,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zstructuredmetadata
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zstreamname = '/app/install'
                """
            )
        else:
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zstructuredmetadata
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zstreamname = "/app/install"
            """
            )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("12"):

                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('KnowledgeC Installed Apps')
                report.start_artifact_report(report_folder, 'Installed Apps')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'App Category',
                    'App Name',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Installed Apps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Installed Apps'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4], row[5]))

                report = ArtifactHtmlReport('KnowledgeC Installed Apps')
                report.start_artifact_report(report_folder, 'Installed Apps')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Installed Apps'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Installed Apps'
                timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data available in Installed Apps')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__locationname,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__displayname,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__fullyformattedaddress,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__city,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__stateorprovince,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__country,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__postalcode_v2,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__subthoroughfare,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__thoroughfare,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__phonenumbers,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__url,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__contentdescription,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelatedcontenturl,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelateduniqueidentifier,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__latitude,
            zstructuredmetadata.z_dklocationapplicationactivitymetadatakey__longitude,
            zsource.zsourceid,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityuuid,
            zsource.zitemid,
            zsource.zsourceid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zobject.zsource = zsource.z_pk
            and zstreamname = '/app/locationActivity'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                            row[17],
                            row[18],
                            row[19],
                            row[20],
                            row[21],
                            row[22],
                            row[23],
                            row[24],
                            row[25],
                            row[26],
                            row[27],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Location Activity')
                report.start_artifact_report(
                    report_folder, 'Location Activity', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'End',
                    'Bundle ID',
                    'Name',
                    'Display Name',
                    'Formatted Address',
                    'City',
                    'State/Province',
                    'Country',
                    'Postal Code',
                    'Subthoroughfare',
                    'Thoroughfare',
                    'Phone Numebers',
                    'URL',
                    'Activity Type',
                    'Content Description',
                    'User Activity Required String',
                    'Content URL',
                    'Unique ID',
                    'Latitude',
                    'Longitude',
                    'Source ID',
                    'Activity UUID',
                    'Item ID',
                    'Source ID',
                    'Day of the Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Location Activity'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Location Activity'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'KnowledgeC Location Activity'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No data available in Location Activity')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zvalueinteger
            when '0' then 'unlocked'
            when '1' then 'locked'
            end,
            (zobject.zenddate - zobject.zstartdate),
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where zstreamname = '/device/isLocked'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Device Locked')
                report.start_artifact_report(
                    report_folder, 'Device Locked', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Is Locked?',
                    'Usage in Seconds',
                    'Day of the Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Device Locked'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Device Locked'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data in KnowledgeC Device Locked')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dknowplayingmetadatakey__album,
            zstructuredmetadata.z_dknowplayingmetadatakey__artist,
            zstructuredmetadata.z_dknowplayingmetadatakey__genre,
            zstructuredmetadata.z_dknowplayingmetadatakey__title,
            zstructuredmetadata.z_dknowplayingmetadatakey__duration,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zstructuredmetadata
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zstreamname =  '/media/nowPlaying'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Media Playing')
                report.start_artifact_report(
                    report_folder, 'Media Playing', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Now Playing Album',
                    'Now Playing Artists',
                    'Playing Genre',
                    'Playing Title',
                    'Now Playing Duration',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Media Playing'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Media Playing'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Media Playing')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__activitytype,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityrequiredstring,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__itemrelateduniqueidentifier,
            zstructuredmetadata.z_dkapplicationactivitymetadatakey__useractivityuuid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate + 978307200, 'unixepoch'),
            datetime(zstructuredmetadata.z_dkapplicationactivitymetadatakey__expirationdate + 978307200, 'unixepoch')
            from
            zobject, zstructuredmetadata
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zstreamname = '/app/activity'
            and (zobject.zvaluestring = 'com.apple.mobilenotes' or zobject.zvaluestring = 'com.apple.Notes')
            """
            )
            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                        )
                    )

                report = ArtifactHtmlReport('KnowledgeC Notes - Activity')
                report.start_artifact_report(report_folder, 'Notes - Activity')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Activity Type',
                    'User Activity Required String',
                    'ID',
                    'Unique ID',
                    'Activity UUID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                    'Expiration Date',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Notes Activity'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Notes Activity'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Notes Activity')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zvalueinteger
            when '0' then 'portrait'
            when '1' then 'landscape'
            else zobject.zvalueinteger
            end,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname = '/display/orientation'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Screen Orientation')
                report.start_artifact_report(
                    report_folder, 'Screen Orientation', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Orientation',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Screen Orientation'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Screen Orientation'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Screen Orientation')

        if version.parse(iOSversion) >= version.parse("11"):
            cursor.execute(
                """
                select
                datetime(zobject.zstartdate+978307200,'unixepoch') ,
                datetime(zobject.zenddate+978307200,'unixepoch'),
                case
                zobject.zvalueinteger
                when '0' then 'unplugged'
                when '1' then 'plugged in'
                end,
                (zobject.zenddate - zobject.zstartdate),
                case zobject.zstartdayofweek
                when '1' then 'sunday'
                when '2' then 'monday'
                when '3' then 'tuesday'
                when '4' then 'wednesday'
                when '5' then 'thursday'
                when '6' then 'friday'
                when '7' then 'saturday'
                end,
                zobject.zsecondsfromgmt/3600,
                datetime(zobject.zcreationdate+978307200,'unixepoch')
                from
                zobject
                where
                zstreamname = '/device/isPluggedIn'
                """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Plugged In')
                report.start_artifact_report(report_folder, 'Plugged In', description)
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Is Plugged In?',
                    'Usage in Seconds',
                    'Day of the Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Plugged In'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Plugged In'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data in KnowledgeC Plugged In')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zstructuredmetadata.z_dksafarihistorymetadatakey__title,
            zobject.zvaluestring,
            zsource.zbundleid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject,
            zstructuredmetadata, zsource
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and  zobject.zsource = zsource.z_pk
            and zstreamname = '/safari/history'
            """
            )
        elif version.parse(iOSversion) == version.parse("11"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            zsource.zbundleid,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zsource
            where zobject.zsource = zsource.z_pk
            and zstreamname = '/safari/history'
            """
            )
        else:
            logfunc("Unsupported version for KnowledgC Safari iOS " + iOSversion)
            return ()

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            if version.parse(iOSversion) >= version.parse("12"):

                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                report = ArtifactHtmlReport('KnowledgeC Safari Browsing')
                report.start_artifact_report(report_folder, 'Safari Browsing')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Title',
                    'URL',
                    'Bundle ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Safari Browsing'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Safari Browsing'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6])
                    )

                report = ArtifactHtmlReport('KnowledgeC Safari Browsing')
                report.start_artifact_report(report_folder, 'Safari Browsing')
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'URL',
                    'Bundle ID',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Safari Browsing'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Safari Browsing'
                timeline(report_folder, tlactivity, data_list, data_headers)

        else:
            logfunc('No data available in Safari Browsing')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            zobject.zvaluestring,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname =  '/siri/ui'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Siri Usage')
                report.start_artifact_report(report_folder, 'Siri Usage', description)
                report.add_script()
                data_headers = (
                    'Start',
                    'App Name',
                    'Weekday',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Siri Usage'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Siri Usage'
                timeline(report_folder, tlactivity, data_list, data_headers)
            else:
                logfunc('No data in KnowledgeC Siri Usage')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where zstreamname = '/app/usage'
            """
            )
        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append(
                    (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                )

            description = ''
            report = ArtifactHtmlReport('KnowledgeC App Usage')
            report.start_artifact_report(report_folder, 'App Usage', description)
            report.add_script()
            data_headers = (
                'Start',
                'End',
                'Bundle ID',
                'Usage in Seconds',
                'Usage in Minutes',
                'Day of the Week',
                'GMT Offset',
                'Entry Creation',
            )
            report.write_artifact_data_table(
                data_headers, data_list, file_found, html_escape=False
            )
            report.end_artifact_report()

            tsvname = 'KnowledgeC App Usage'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'KnowledgeC App Usage'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data in KnowledgeC App Usage')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where zstreamname = '/system/userWakingEvent'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3], row[4]))

                description = ''
                report = ArtifactHtmlReport('KnowledgeC User Waking Event')
                report.start_artifact_report(
                    report_folder, 'User Waking Event', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC User Waking Events'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC User Waking Events'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in User Waking Event')

        if version.parse(iOSversion) >= version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            zstructuredmetadata .z_dkdigitalhealthmetadatakey__webdomain,
            zstructuredmetadata .z_dkdigitalhealthmetadatakey__webpageurl,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject, zstructuredmetadata
            where zobject.zstructuredmetadata = zstructuredmetadata.z_pk
            and zstreamname = '/app/webUsage'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Web Usage')
                report.start_artifact_report(report_folder, 'Web Usage', description)
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'App Name',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Domain',
                    'URL',
                    'Day of the Wekk',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Web Usage'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Web Usage'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Web Usage')

        if version.parse(iOSversion) < version.parse("12"):
            cursor.execute(
                """
            select
            datetime(zobject.zstartdate+978307200,'unixepoch'),
            datetime(zobject.zenddate+978307200,'unixepoch'),
            zobject.zvaluestring,
            (zobject.zenddate - zobject.zstartdate),
            (zobject.zenddate - zobject.zstartdate)/60.00,
            case zobject.zstartdayofweek
            when '1' then 'sunday'
            when '2' then 'monday'
            when '3' then 'tuesday'
            when '4' then 'wednesday'
            when '5' then 'thursday'
            when '6' then 'friday'
            when '7' then 'saturday'
            end,
            zobject.zsecondsfromgmt/3600,
            datetime(zobject.zcreationdate+978307200,'unixepoch')
            from
            zobject
            where
            zstreamname = '/widgets/viewed'
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = ''
                report = ArtifactHtmlReport('KnowledgeC Widgets Viewed')
                report.start_artifact_report(
                    report_folder, 'Widgets Viewed', description
                )
                report.add_script()
                data_headers = (
                    'Start',
                    'End',
                    'Bundle ID',
                    'Usage in Seconds',
                    'Usage in Minutes',
                    'Day of Week',
                    'GMT Offset',
                    'Entry Creation',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'KnowledgeC Widgets Viewed'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'KnowledgeC Widgets Viewed'
                timeline(report_folder, tlactivity, data_list, data_headers)

            else:
                logfunc('No data available in Widgets Viewed')



from helpers.db import open_sqlite_db_readonly
from helpers import timeline, tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class DataUsageProcessB(ab.Artifact):

    _name = 'Data Usage Process'
    _search_dirs = '**/DataUsage.sqlite'
    _category = 'Data Usage'
    _web_icon = WebIcon.WIFI

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(zprocess.ztimestamp+ 978307200, 'unixepoch'),
        datetime(zprocess.zfirsttimestamp + 978307200, 'unixepoch'),
        zprocess.zprocname,
        zprocess.zbundlename
        from zprocess
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3]))

            report = ArtifactHtmlReport('Data Usage')
            report.start_artifact_report(report_folder, 'Data Usage Process')
            report.add_script()
            data_headers = (
                'Timestamp',
                'Process First Timestamp',
                'Process Name',
                'Bundle ID',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Data Usage Process'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Data Usage Process'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Data Usage available')

        db.close()



import string

from helpers import tsv
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class DiscordAcct(ab.Artifact):

    _name = 'Discord Account'
    _search_dirs = (
        '*/var/mobile/Containers/Data/Application/*/Documents/mmkv/mmkv.default'
    )
    _category = 'Discord'
    _web_icon = WebIcon.USER

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        searchlist = []
        for file_found in files_found:
            file_found = str(file_found)

            for s in strings(file_found):
                # print(type(s))
                # print(s)
                searchlist.append(
                    str(s),
                )

            counter = 0
            data_list = []
            for x in searchlist:
                counter += 1
                if 'user_id_cache' in x:
                    # print(x)
                    wf = searchlist[counter].split('"')
                    try:
                        data_list.append(('USER_ID_CACHE', wf[1]))
                    except:
                        pass

                if 'email_cache' in x:
                    # print(x)
                    wfa = searchlist[counter].split('"')
                    try:
                        data_list.append(('EMAIL_CACHE', wfa[1]))
                    except:
                        pass

        if len(data_list) > 0:
            report = ArtifactHtmlReport('Discord Account')
            report.start_artifact_report(report_folder, 'Discord Account')
            report.add_script()
            data_headers = ('Key', 'Value')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Discord Account'
            tsv(report_folder, data_headers, data_list, tsvname)


def strings(filename, min=4):
    with open(filename, errors="ignore") as f:  # Python 3.x
        # with open(filename, "rb") as f:           # Python 2.x
        result = ""
        for c in f.read():
            if c in string.printable:
                result += c
                continue
            if len(result) >= min:
                yield result
            result = ""
        if len(result) >= min:  # catch result at EOF
            yield result



import glob
import json
import os
import pathlib
import plistlib
import sqlite3
import stat
import sys

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import (
    does_table_exist,
    is_platform_windows,
    kmlgen,
    logfunc,
    open_sqlite_db_readonly,
    timeline,
    tsv,
)

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class LocationDAllB(ab.Artifact):

    _name = 'App Harvest'
    _search_dirs = '**/cache_encryptedB.db'
    _category = 'Locations'

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) >= version.parse("11"):
            logfunc(
                "Unsupported version for LocationD App Harvest on iOS " + iOSversion
            )
        else:
            logfunc(iOSversion)
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp + 978307200,'unixepoch'),
            bundleid,
            altitude,
            horizontalaccuracy,
            verticalaccuracy,
            state,
            age,
            routinemode,
            locationofinteresttype,
            latitude,
            longitude,
            speed,
            course,
            confidence
            from appharvest
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD App Harvest')
                report.start_artifact_report(report_folder, 'App Harvest', description)
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Bundle ID',
                    'Altitude',
                    'Horizontal Accuracy',
                    'Vertical Accuracy',
                    'State',
                    'Age',
                    'Routine Mode',
                    'Location of Interest Type',
                    'Latitude',
                    'Longitude',
                    'Speed',
                    'Course',
                    'Confidence',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD Cell App Harvest'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'LocationD Cell App Harvest'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'LocationD Cell App Harvest'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No data available for LocationD App Harvest')

        if does_table_exist(db, "cdmacelllocation"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp + 978307200,'unixepoch'),
            mcc,
            sid,
            nid,
            bsid,
            zoneid,
            bandclass,
            channel,
            pnoffset,
            altitude,
            speed,
            course,
            confidence,
            horizontalaccuracy,
            verticalaccuracy,
            latitude,
            longitude
            from cdmacelllocation
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                            row[15],
                            row[16],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD CDMA Location')
                report.start_artifact_report(
                    report_folder, 'CDMA Location', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'MCC',
                    'SID',
                    'NID',
                    'BSID',
                    'ZONEID',
                    'BANDCLASS',
                    'Channel',
                    'PNOFFSET',
                    'Altitude',
                    'Speed',
                    'Course',
                    'Confidence',
                    'Horizontal Accuracy',
                    'Vertical Accuracy',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD CDMA Location'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'LocationD CDMA Location'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'LocationD CDMA Location'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No data available for LocationD CDMA Location')

        if does_table_exist(db, "celllocation"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp + 978307200,'unixepoch'),
            mcc,
            mnc,
            lac,
            ci,
            uarfcn,
            psc,
            altitude,
            speed,
            course,
            confidence,
            horizontalaccuracy,
            verticalaccuracy,
            latitude,
            longitude
            from celllocation
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                            row[14],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD Cell Location')
                report.start_artifact_report(
                    report_folder, 'Cell Location', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'MCC',
                    'MNC',
                    'LAC',
                    'CI',
                    'UARFCN',
                    'PSC',
                    'Altitude',
                    'Speed',
                    'Course',
                    'Confidence',
                    'Horizontal Accuracy',
                    'Vertical Accuracy',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD Cell Location'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'LocationD Cell Location'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'LocationD Cell Location'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No data available for LocationD Cell Location')

        if does_table_exist(db, "ltecelllocation"):
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(timestamp + 978307200,'unixepoch'),
            mcc,
            mnc,
            ci,
            uarfcn,
            pid,
            altitude,
            speed,
            course,
            confidence,
            horizontalaccuracy,
            verticalaccuracy,
            latitude,
            longitude
            from ltecelllocation
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                            row[10],
                            row[11],
                            row[12],
                            row[13],
                        )
                    )

                description = ''
                report = ArtifactHtmlReport('LocationD LTE Location')
                report.start_artifact_report(report_folder, 'LTE Location', description)
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'MCC',
                    'MNC',
                    'CI',
                    'UARFCN',
                    'PID',
                    'Altitude',
                    'Speed',
                    'Course',
                    'Confidence',
                    'Horizontal Accuracy',
                    'Vertical Accuracy',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'LocationD LTE Location'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'LocationD LTE Location'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'LocationD LTE Location'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
            else:
                logfunc('No data available for LocationD LTE Location')

        cursor = db.cursor()
        cursor.execute(
            """
        select
        datetime(timestamp + 978307200,'unixepoch'),
        mac,
        channel,
        infomask,
        speed,
        course,
        confidence,
        score,
        reach,
        horizontalaccuracy,
        verticalaccuracy,
        latitude,
        longitude
        from wifilocation
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                        row[9],
                        row[10],
                        row[11],
                        row[12],
                    )
                )

            description = ''
            report = ArtifactHtmlReport('LocationD WiFi Location')
            report.start_artifact_report(report_folder, 'WiFi Location', description)
            report.add_script()
            data_headers = (
                'Timestamp',
                'MAC',
                'Channel',
                'Infomask',
                'Speed',
                'Course',
                'Confidence',
                'Score',
                'Reach',
                'Horizontal Accuracy',
                'Vertical Accuracy',
                'Latitude',
                'Longitude',
            )
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'LocationD WiFi Location'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'LocationD WiFi Location'
            timeline(report_folder, tlactivity, data_list, data_headers)

            kmlactivity = 'LocationD WiFi Location'
            kmlgen(report_folder, kmlactivity, data_list, data_headers)
        else:
            logfunc('No data available for LocationD WiFi Location')



import plistlib

from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv

from artifacts.Artifact import Artifact


class DataArk(ab.Artifact):

    _name = 'Data Ark'
    _search_dirs = '**/Library/Lockdown/data_ark.plist'
    _category = 'IOS Build'
    _web_icon = WebIcon.GIT_COMMIT

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        file_found = str(files_found[0])
        with open(file_found, "rb") as fp:
            pl = plistlib.load(fp)
            for key, val in pl.items():
                data_list.append((key, val))

                if key == "-DeviceName":
                    logdevinfo(f"Device name: {val}")
                if key == "-TimeZone":
                    logdevinfo(f"Timezone per Data Ark: {val}")
                if key == "com.apple.iTunes.backup-LastBackupComputerName":
                    logdevinfo(f"Last backup computer name: {val}")
                if key == ("com.apple.iTunes.backup-LastBackupComputerType"):
                    logdevinfo(f"Last backup computer type: {val}")

        report = ArtifactHtmlReport('Data Ark')
        report.start_artifact_report(report_folder, 'Data Ark')
        report.add_script()
        data_headers = ('Key', 'Values')
        report.write_artifact_data_table(data_headers, data_list, file_found)
        report.end_artifact_report()

        tsvname = 'Data Ark'
        tsv(report_folder, data_headers, data_list, tsvname)



import os
import pathlib
import plistlib
import sys

import biplist
import nska_deserialize as nd
from helpers import timeline, tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class AppItunesMetadata(ab.Artifact):

    _name = "App iTunes Metadata"
    _search_dirs = ('**/iTunesMetadata.plist', '**/BundleMetadata.plist')
    _category = 'Installed Apps'
    _web_icon = WebIcon.PACKAGE

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('iTunesMetadata.plist'):
                with open(file_found, "rb") as fp:
                    if sys.version_info >= (3, 9):
                        plist = plistlib.load(fp)
                    else:
                        plist = biplist.readPlist(fp)

                purchasedate = plist.get('com.apple.iTunesStore.downloadInfo', {}).get(
                    'purchaseDate', ''
                )
                bundleid = plist.get('softwareVersionBundleId', '')
                itemname = plist.get('itemName', '')
                artistname = plist.get('artistName', '')
                versionnum = plist.get('bundleShortVersionString', '')
                downloadedby = (
                    plist.get('com.apple.iTunesStore.downloadInfo', {})
                    .get('accountInfo', {})
                    .get('AppleID', '')
                )
                genre = plist.get('genre', '')
                factoryinstall = plist.get('isFactoryInstall', '')
                appreleasedate = plist.get('releaseDate', '')
                sourceapp = plist.get('sourceApp', '')
                sideloaded = plist.get('sideLoadedDeviceBasedVPP', '')
                variantid = plist.get('variantID', '')

                p = pathlib.Path(file_found)
                parent = p.parent

                itunes_metadata_path = os.path.join(parent, "BundleMetadata.plist")
                if os.path.exists(itunes_metadata_path):
                    with open(itunes_metadata_path, 'rb') as f:
                        deserialized_plist = nd.deserialize_plist(f)
                        install_date = deserialized_plist.get('installDate', '')
                else:
                    install_date = ''

                data_list.append(
                    (
                        install_date,
                        purchasedate,
                        bundleid,
                        itemname,
                        artistname,
                        versionnum,
                        downloadedby,
                        genre,
                        factoryinstall,
                        appreleasedate,
                        sourceapp,
                        sideloaded,
                        variantid,
                        parent,
                    )
                )

        if len(data_list) > 0:
            fileloc = 'See source file location column'
            description = 'iTunes & Bundle ID Metadata contents for apps'
            report = ArtifactHtmlReport('Apps - Itunes & Bundle Metadata')
            report.start_artifact_report(
                report_folder, 'Apps - Itunes Metadata', description
            )
            report.add_script()
            data_headers = (
                'Installed Date',
                'App Purchase Date',
                'Bundle ID',
                'Item Name',
                'Artist Name',
                'Version Number',
                'Downloaded by',
                'Genre',
                'Factory Install',
                'App Release Date',
                'Source App',
                'Sideloaded?',
                'Variant ID',
                'Source File Location',
            )
            report.write_artifact_data_table(data_headers, data_list, fileloc)
            report.end_artifact_report()

            tsvname = 'Apps - Itunes Bundle Metadata'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Apps - Itunes Bundle Metadata'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No data on Apps - Itunes Bundle Metadata')



import datetime
import glob
import os

from html_report.artifact_report import ArtifactHtmlReport
from helpers.parsers import cclparser
from helpers import logfunc

from artifacts.Artifact import Artifact


class NotificationsXI(ab.Artifact):
    _name = 'iOS Notifications'
    _search_dirs = '*PushStore*'
    _report_section = 'Notifications'

    def get(files_found, seeker):
        pathfound = 0
        count = 0
        notdircount = 0
        exportedbplistcount = 0
        unix = datetime.datetime(1970, 1, 1)  # UTC
        cocoa = datetime.datetime(2001, 1, 1)  # UTC
        delta = cocoa - unix

        __location__ = Path.cwd().resolve()

        pathfound = str(files_found[0])
        # logfunc(pathfound)
        if pathfound == 0:
            logfunc("No PushStore directory located")
        else:
            # logfunc('Pathfound was found')
            folder = self.report_folder
            # logfunc("Processing:")
            for filename in glob.iglob(pathfound + "/**", recursive=True):
                if os.path.isfile(filename):  # filter dirs
                    file_name = os.path.splitext(os.path.basename(filename))[0]
                    # get extension and iterate on those files
                    # file_extension = os.path.splitext(filename)
                    # logfunc(file_extension)
                    # create directory
                    if filename.endswith("pushstore"):
                        # create directory where script is running from
                        # logfunc(filename)  # full path
                        notdircount = notdircount + 1
                        # logfunc (os.path.basename(file_name)) # filename with  no extension
                        openplist = os.path.basename(
                            os.path.normpath(filename)
                        )  # filename with extension
                        # logfunc (openplist)
                        # bundlepath = (os.path.basename(os.path.dirname(filename)))# previous directory
                        bundlepath = file_name
                        appdirect = folder + bundlepath
                        # logfunc(appdirect)
                        os.makedirs(appdirect)

                        # open the plist
                        p = open(filename, "rb")
                        plist = cclparser.load(p)
                        plist2 = plist["$objects"]

                        long = len(plist2)
                        # logfunc (long)
                        h = open(
                            appdirect + "/DeliveredNotificationsReport.html", "w"
                        )  # write report
                        h.write("<html><body>")
                        h.write("<h2>iOS Delivered Notifications Triage Report </h2>")
                        h.write(filename)
                        h.write("<br/>")
                        h.write(
                            "<style> table, td {border: 1px solid black; border-collapse: collapse;}tr:nth-child(even) {background-color: # f2f2f2;} .table th { background: # 888888; color: # ffffff}.table.sticky th{ position:sticky; top: 0; }</style>"
                        )
                        h.write("<br/>")

                        h.write('<button onclick="hideRows()">Hide rows</button>')
                        h.write('<button onclick="showRows()">Show rows</button>')

                        f = open(os.path.join(__location__, "script.txt"), "r")
                        for line in f:
                            h.write(line)
                        f.close()

                        h.write("<br>")
                        h.write('<table name="hide">')
                        h.write('<tr name="hide">')
                        h.write("<th>Data type</th>")
                        h.write("<th>Value</th>")
                        h.write("</tr>")

                        h.write('<tr name="hide">')
                        h.write("<td>Plist</td>")
                        h.write("<td>Initial Values</td>")
                        h.write("</tr>")

                        test = 0
                        for i in range(0, long):
                            try:
                                if plist2[i]["$classes"]:
                                    h.write('<tr name="hide">')
                                    h.write("<td>$classes</td>")
                                    ob6 = str(plist2[i]["$classes"])
                                    h.write("<td>")
                                    h.write(str(ob6))
                                    h.write("</td>")
                                    h.write("</tr>")
                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["$class"]:
                                    h.write('<tr name="hide">')
                                    h.write("<td>$class</td>")
                                    ob5 = str(plist2[i]["$class"])
                                    h.write("<td>")
                                    h.write(str(ob5))
                                    h.write("</td>")
                                    h.write("</tr>")
                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["NS.keys"]:
                                    h.write('<tr name="hide">')
                                    h.write("<td>NS.keys</td>")
                                    ob0 = str(plist2[i]["NS.keys"])
                                    h.write("<td>")
                                    h.write(str(ob0))
                                    h.write("</td>")
                                    h.write("</tr>")
                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["NS.objects"]:
                                    ob1 = str(plist2[i]["NS.objects"])
                                    h.write('<tr name="hide">')
                                    h.write("<td>NS.objects</td>")
                                    h.write("<td>")
                                    h.write(str(ob1))
                                    h.write("</td>")
                                    h.write("</tr>")

                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["NS.time"]:
                                    dia = str(plist2[i]["NS.time"])
                                    dias = dia.rsplit(".", 1)[0]
                                    timestamp = (
                                        datetime.datetime.fromtimestamp(int(dias))
                                        + delta
                                    )
                                    # logfunc (timestamp)

                                    h.write("<tr>")
                                    h.write("<td>Time UTC</td>")
                                    h.write("<td>")
                                    h.write(str(timestamp))
                                    # h.write(str(plist2[i]['NS.time']))
                                    h.write("</td>")
                                    h.write("</tr>")

                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["NS.base"]:
                                    ob2 = str(plist2[i]["NS.objects"])
                                    h.write('<tr name="hide">')
                                    h.write("<td>NS.base</td>")
                                    h.write("<td>")
                                    h.write(str(ob2))
                                    h.write("</td>")
                                    h.write("</tr>")

                                    test = 1
                            except:
                                pass
                            try:
                                if plist2[i]["$classname"]:
                                    ob3 = str(plist2[i]["$classname"])
                                    h.write('<tr name="hide">')
                                    h.write("<td>$classname</td>")
                                    h.write("<td>")
                                    h.write(str(ob3))
                                    h.write("</td>")
                                    h.write("</tr>")

                                    test = 1
                            except:
                                pass

                            try:
                                if test == 0:
                                    if (plist2[i]) == "AppNotificationMessage":
                                        h.write("</table>")
                                        h.write("<br>")
                                        h.write("<table>")
                                        h.write("<tr>")
                                        h.write("<th>Data type</th>")
                                        h.write("<th>Value</th>")
                                        h.write("</tr>")

                                        h.write('<tr name="hide">')
                                        h.write("<td>ASCII</td>")
                                        h.write("<td>" + str(plist2[i]) + "</td>")
                                        h.write("</tr>")

                                    else:
                                        if plist2[i] in notiparams:
                                            h.write('<tr name="hide">')
                                            h.write("<td>ASCII</td>")
                                            h.write("<td>" + str(plist2[i]) + "</td>")
                                            h.write("</tr>")
                                        elif plist2[i] == " ":
                                            h.write('<tr name="hide">')
                                            h.write("<td>Null</td>")
                                            h.write("<td>" + str(plist2[i]) + "</td>")
                                            h.write("</tr>")
                                        else:
                                            h.write("<tr>")
                                            h.write("<td>ASCII</td>")
                                            h.write("<td>" + str(plist2[i]) + "</td>")
                                            h.write("</tr>")

                            except:
                                pass

                            test = 0

                            # h.write('test')

                        for dict in plist2:
                            liste = dict
                            types = type(liste)
                            # logfunc (types)
                            try:
                                for k, v in liste.items():
                                    if k == "NS.data":
                                        chk = str(v)
                                        reduced = chk[2:8]
                                        # logfunc (reduced)
                                        if reduced == "bplist":
                                            count = count + 1
                                            binfile = open(
                                                "./"
                                                + appdirect
                                                + "/incepted"
                                                + str(count)
                                                + ".bplist",
                                                "wb",
                                            )
                                            binfile.write(v)
                                            binfile.close()

                                            procfile = open(
                                                "./"
                                                + appdirect
                                                + "/incepted"
                                                + str(count)
                                                + ".bplist",
                                                "rb",
                                            )
                                            secondplist = cclparser.load(procfile)
                                            secondplistint = secondplist["$objects"]
                                            # logfunc("Bplist processed and exported.")
                                            exportedbplistcount = (
                                                exportedbplistcount + 1
                                            )
                                            h.write('<tr name="hide">')
                                            h.write("<td>NS.data</td>")
                                            h.write("<td>")
                                            h.write(str(secondplistint))
                                            h.write("</td>")
                                            h.write("</tr>")

                                            procfile.close()
                                            count = 0
                                        else:
                                            h.write('<tr name="hide">')
                                            h.write("<td>NS.data</td>")
                                            h.write("<td>")
                                            h.write(str(secondplistint))
                                            h.write("</td>")
                                            h.write("</tr>")
                            except:
                                pass
                        h.close()
                    elif "AttachmentsList" in file_name:
                        test = 0  # future development

        path = self.report_folder
        level2, level1 = os.path.split(path)

        # final = level2+'/'+level1
        dict = {}
        files = os.listdir(path)
        for name in files:
            try:
                size = os.path.getsize(
                    f"{path}{name}/DeliveredNotificationsReport.html"
                )
                key = f'<a href = "{level2}/{name}/DeliveredNotificationsReport.html" style = "color:blue" target="content">{name}</a>'
                dict[key] = size
            except NotADirectoryError as nade:
                logfunc(nade)
                pass

        data_list = []
        for k, v in dict.items():
            v = v / 1000
            # logfunc(f'{k} -> {v}')
            data_list.append((k, v))

        location = pathfound
        description = 'iOS <= 11 Notifications'
        report = ArtifactHtmlReport('iOS Notificatons')
        report.start_artifact_report(
            self.report_folder, 'iOS Notifications', description
        )
        report.add_script()
        data_headers = ('Bundle GUID', 'Reports Size')
        report.write_artifact_data_table(
            data_headers, data_list, location, html_escape=False
        )
        report.end_artifact_report()

        logfunc("Total notification directories processed:" + str(notdircount))
        logfunc("Total exported bplists from notifications:" + str(exportedbplistcount))
        if notdircount == 0:
            logfunc("No notifications located.")


NotificationParams = [
    'AppNotificationAttachments',
    'AppNotificationBadgeNumber',
    'AppNotificationContentAvailable',
    'AppNotificationCreationDate',
    'AppNotificationIdentifier',
    'AppNotificationMessage',
    'AppNotificationMessageLocalizationArguments',
    'AppNotificationMessageLocazationKey',
    'AppNotificationMutableContent',
    'AppNotificationSummaryArgument',
    'AppNotificationSummaryArgumentCount',
    'AppNotificationTitle',
    'BadgeApplicationIcon',
    'CriticalAlertSound',
    'HasDefaultActionKey',
    'Header',
    'SBSPushStoreNotificationCategoryKey',
    'SBSPushStoreNotificationThreadKey',
    'SchemaVersion',
    'ShouldAuthenticateDefaultAction',
    'ShouldBackgroundDefaultAction',
    'ShouldHideDate',
    'ShouldHideTime',
    'ShouldIgnoreDoNotDisturb',
    'ShouldIgnoreDowntime',
    'ShouldPlaySound',
    'ShouldPresentAlert',
    'ShouldPreventNotificationDismissalAfterDefaultAction',
    'ShouldSuppressScreenLightUp',
    'ShouldSuppressSyncDismissalWhenRemoved',
    'ShouldUseRequestIdentifierForDismissalSync',
    'SoundMaximumDuration',
    'SoundShouldIgnoreRingerSwitch',
    'SoundShouldRepeat',
    'ToneAlertType',
    'ToneFileName',
    'ToneMediaLibraryItemIdentifier',
    'TriggerRepeatInterval',
    'TriggerRepeats',
    'TriggerTimeInterval',
    'UNNotificationAlertDestination',
    'UNNotificationCarPlayDestination',
    'UNNotificationDefaultDestinations',
    'UNNotificationLockScreenDestination',
    'UNNotificationNotificationCenterDestination',
    'UNNotificationTriggerType',
    'UNNotificationUserInfo',
]



import os
import pathlib
import re
from datetime import datetime

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, tsv

from artifacts.Artifact import Artifact


class MobileActivationLogs(ab.Artifact):
    _name = 'Mobile Activation Logs'
    _search_dirs = '**/mobileactivationd.log*'
    _category = 'Mobile Activation Logs'

    def get(self, files_found, seeker):
        data_list = []
        data_list_info = []

        source_files = []
        for file_found in files_found:
            file_found = str(file_found)
            if file_found.startswith('\\\\?\\'):
                file_name = pathlib.Path(file_found[4:]).name
                source_files.append(file_found[4:])
            else:
                file_name = pathlib.Path(file_found).name
                source_files.append(file_found)

            with open(file_found, 'r') as fp:
                data = fp.readlines()
                linecount = 0
                hitcount = 0
                activationcount = 0

                for line in data:
                    linecount += 1
                    date_filter = re.compile(
                        r'(([A-Za-z]+[\s]+([a-zA-Z]+[\s]+[0-9]+)[\s]+([0-9]+\:[0-9]+\:[0-9]+)[\s]+([0-9]{4}))([\s]+[\[\d\]]+[\s]+[\<a-z\>]+[\s]+[\(\w\)]+[\s]+[A-Z]{2}\:[\s]+)([main\:\s]*.*)$)'
                    )
                    line_match = re.match(date_filter, line)

                    if line_match:
                        date_time = line_match.group(3, 5, 4)
                        conv_time = ' '.join(date_time)
                        dtime_obj = datetime.strptime(conv_time, '%b %d %Y %H:%M:%S')
                        ma_datetime = str(dtime_obj)
                        values = line_match.group(7)

                        if 'perform_data_migration' in values:
                            hitcount += 1
                            upgrade_match = re.search(
                                (
                                    r'((.*)(Upgrade\s+from\s+[\w]+\s+to\s+[\w]+\s+detected\.$))'
                                ),
                                values,
                            )
                            if upgrade_match:
                                upgrade = upgrade_match.group(3)
                                data_list.append((ma_datetime, upgrade, file_name))

                        if (
                            '____________________ Mobile Activation Startup _____________________'
                            in values
                        ):
                            activationcount += 1
                            ma_startup_line = str(linecount)
                            ma_startup = (
                                f'Mobile Activation Startup at line: {ma_startup_line}'
                            )
                            data_list.append((ma_datetime, ma_startup, file_name))

                    upgrade_entries = f'Found {hitcount} Upgrade entries in {file_name}'
                    boot_entries = f'Found {activationcount} Mobile Activation entries in {file_name}'
                data_list_info.append((boot_entries, upgrade_entries))

        report = ArtifactHtmlReport('Mobile Activation Logs')
        report.start_artifact_report(report_folder, 'Mobile Activation Logs')
        report.add_script()
        data_headers_info = ('Mobile Activation', 'Upgrade')
        data_headers = ('Datetime', 'Event', 'Log Name')

        source_files_found = ', '.join(source_files)

        report.write_artifact_data_table(
            data_headers_info,
            data_list_info,
            source_files_found,
            cols_repeated_at_bottom=False,
        )
        report.write_artifact_data_table(
            data_headers, data_list, source_files_found, True, False
        )
        report.end_artifact_report()

        tsvname = 'Mobile Activation Logs'
        tsv(report_folder, data_headers, data_list, tsvname)
        tsv(report_folder, data_headers_info, data_list_info, tsvname)



import glob
import os
import pathlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class TitleAppNetDB(ab.Artifact):
    _name = 'Tile App Account Information'
    _search_dirs = '*/private/var/mobile/Containers/Shared/AppGroup/*/com.thetxleapp.tile-TileNetworkDB.sqlite*'
    _category = 'Accounts'

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('tile-TileNetworkDB.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        datetime(ZREGISTRATION_TIMESTAMP,'unixepoch','31 years'),
        ZEMAIL,
        ZFULL_NAME,
        ZMOBILE_PHONE
        FROM
        ZTILENTITY_USER
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append((row[0], row[1], row[2], row[3]))

                description = ''
                report = ArtifactHtmlReport('Tile App - Account Information')
                report.start_artifact_report(
                    report_folder, 'Tile App Account Information', description
                )
                report.add_script()
                data_headers = (
                    'Registration Timestamp',
                    'Email',
                    'Full Name',
                    'Mobile Phone Number',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

            tsvname = 'Tile App Account Information'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Tile App Account Information'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Tile App DB account data available')

        db.close()



import io

import nska_deserialize as nd
from helpers import tsv
from helpers.db import open_sqlite_db_readonly
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class ApplicationState(ab.Artifact):

    _name = 'Application State'
    _search_dirs = '**/applicationState.db'
    _category = 'Installed Apps'
    _web_icon = WebIcon.PACKAGE

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        file_found = str(files_found[0])
        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        select ait.application_identifier as ai, kvs.value as compat_info,
        (SELECT kvs.value from kvs left join application_identifier_tab on application_identifier_tab.id = kvs.application_identifier
        left join key_tab on kvs.key = key_tab.id
        WHERE key_tab.key='XBApplicationSnapshotManifest' and kvs.key = key_tab.id
        and application_identifier_tab.id = ait.id
        ) as snap_info
        from kvs
        left join application_identifier_tab ait on ait.id = kvs.application_identifier
        left join key_tab on kvs.key = key_tab.id
        where key_tab.key='compatibilityInfo'
        order by ait.id
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        if usageentries > 0:
            data_list = []
            snap_info_list = []
            for row in all_rows:
                # bundleid = str(row[0])
                plist_file_object = io.BytesIO(row[1])
                try:
                    plist = nd.deserialize_plist(plist_file_object)

                    if type(plist) is dict:
                        var1 = plist.get('bundleIdentifier', '')
                        var2 = plist.get('bundlePath', '')
                        var3 = plist.get('sandboxPath', '')
                        data_list.append((var1, var2, var3))
                        if row[2]:
                            snap_info_list.append((var1, var2, var3, row[2]))
                    else:
                        logfunc(
                            f'For {row[0]} Unexpected type '
                            f'{str(type(plist))} found as plist '
                            f'root, can\'t process'
                        )
                except (
                    nd.DeserializeError,
                    nd.biplist.NotBinaryPlistException,
                    nd.biplist.InvalidPlistException,
                    nd.ccl_bplist.BplistError,
                    ValueError,
                    TypeError,
                    OSError,
                    OverflowError,
                ) as ex:
                    logfunc(
                        f'Failed to read plist for {row[0]}, error was: ' f'{str(ex)}'
                    )

            report = ArtifactHtmlReport('Application State')
            report.start_artifact_report(report_folder, 'Application State DB')
            report.add_script()
            data_headers = ('Bundle ID', 'Bundle Path', 'Sandbox Path')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Application State'
            tsv(report_folder, data_headers, data_list, tsvname)
        else:
            logfunc('No Application State data available')

        db.close()



import glob
import os
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from packaging import version
from helpers import is_platform_windows, kmlgen, open_sqlite_db_readonly, timeline, tsv

import artifacts.artGlobals

from artifacts.Artifact import Artifact


class RoutineDLocations(ab.Artifact):
    _name = 'RoutineD ZRTCLLOCATIONMO'
    _search_dirs = '**/com.apple.routined/Cache.sqlite*'
    _category = 'Locations'

    def get(self, files_found, seeker):
        iOSversion = artifacts.artGlobals.versionf
        if version.parse(iOSversion) < version.parse("10"):
            logfunc(
                "Unsupported version for RoutineD Locations Cache.sqlite on iOS "
                + iOSversion
            )
        else:
            for file_found in files_found:
                file_found = str(file_found)

                if file_found.endswith('Cache.sqlite'):
                    break

            db = open_sqlite_db_readonly(file_found)
            cursor = db.cursor()
            cursor.execute(
                """
            select
            datetime(ztimestamp + 978307200, 'unixepoch'),
            zaltitude,
            zcourse,
            zspeed,
            zspeed*2.23694,
            zspeed*3.6,
            zhorizontalaccuracy,
            zverticalaccuracy,
            zlatitude,
            zlongitude
            from
            zrtcllocationmo
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (
                            row[0],
                            row[1],
                            row[2],
                            row[3],
                            row[4],
                            row[5],
                            row[6],
                            row[7],
                            row[8],
                            row[9],
                        )
                    )

                description = 'Granular location data (~ 1 week)'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD ZRTCLLOCATIONMO', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Altitude',
                    'Course',
                    'Speed (M/S)',
                    'Speed (MPH)',
                    'Speed (KMPH)',
                    'Horizontal Accuracy',
                    'Vertical Accuracy',
                    'Latitude',
                    'Longitude',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD ZRTCLLOCATIONMO'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD ZRTCLLOCATIONMO'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD ZRTCLLOCATIONMO'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No RoutineD ZRTCLLOCATIONMO data available')

            cursor.execute(
                """
            select
            datetime(zdate + 978307200, 'unixepoch'),
            zsource,
            zlatitude,
            zlongitude
            from
            zrthintmo
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2], row[3]))

                description = 'Semi-granular location data (~ 1 week)'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD ZRTHINTMO', description
                )
                report.add_script()
                data_headers = ('Timestamp', 'Source', 'Latitude', 'Longitude')
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD ZRTHINTMO'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD ZRTHINTMO'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD ZRTHINTMO'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No RoutineD ZRTHINTMO data available')

            cursor.execute(
                """
            select
            datetime(zentrydate + 978307200, 'unixepoch'),
            datetime(zexitdate + 978307200, 'unixepoch'),
            datetime(zdetectiondate + 978307200, 'unixepoch'),
            (zexitdate-zentrydate)/60.00,
            ztype,
            zlocationlatitude,
            zlocationlongitude,
            zlocationuncertainty
            from
            zrtvisitmo
            """
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)
            data_list = []
            if usageentries > 0:
                for row in all_rows:
                    data_list.append(
                        (row[0], row[1], row[2], row[3], row[4], row[5], row[6], row[7])
                    )

                description = 'Visit locations'
                report = ArtifactHtmlReport('Locations')
                report.start_artifact_report(
                    report_folder, 'RoutineD ZRTVISITMO', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Exit Timestamp',
                    'Detection Timestamp',
                    'Visit Time (Minutes)',
                    'Type',
                    'Latitude',
                    'Longitude',
                    'Uncertainty',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'RoutineD ZRTVISITMO'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'RoutineD ZRTVISITMO'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'RoutineD ZRTVISITMO'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)

            else:
                logfunc('No RoutineD ZRTVISITMO data available')



import io
import os

import nska_deserialize as nd
from helpers.db import open_sqlite_db_readonly
from helpers import tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class CloudKitParticipants(ab.Artifact):

    _name = 'CloudKit Participants'
    _search_dirs = '*NoteStore.sqlite*'
    _category = 'CloudKit'
    _web_icon = WebIcon.USER

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):

        user_dictionary = {}

        for file_found in files_found:
            file_found = str(file_found)

            # Can add a separate section for each file this information is found in.
            # This is for Apple Notes.
            if file_found.endswith('NoteStore.sqlite'):
                db = open_sqlite_db_readonly(file_found)
                cursor = db.cursor()
                cursor.execute(
                    """
                SELECT Z_PK, ZSERVERSHAREDATA
                FROM
                ZICCLOUDSYNCINGOBJECT
                WHERE
                ZSERVERSHAREDATA NOT NULL
                """
                )

                all_rows = cursor.fetchall()
                for row in all_rows:

                    filename = os.path.join(
                        report_folder, 'zserversharedata_' + str(row[0]) + '.bplist'
                    )
                    output_file = open(filename, "wb")
                    output_file.write(row[1])
                    output_file.close()

                    deserialized_plist = nd.deserialize_plist(io.BytesIO(row[1]))
                    for item in deserialized_plist:
                        if 'Participants' in item:
                            for participant in item['Participants']:
                                record_id = participant['UserIdentity']['UserRecordID'][
                                    'RecordName'
                                ]
                                email_address = participant['UserIdentity'][
                                    'LookupInfo'
                                ]['EmailAddress']
                                phone_number = participant['UserIdentity'][
                                    'LookupInfo'
                                ]['PhoneNumber']
                                first_name = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.givenName']
                                middle_name = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.middleName']
                                last_name = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.familyName']
                                name_prefix = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.namePrefix']
                                name_suffix = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.nameSuffix']
                                nickname = participant['UserIdentity'][
                                    'NameComponents'
                                ]['NS.nameComponentsPrivate']['NS.nickname']

                                user_dictionary[record_id] = [
                                    record_id,
                                    email_address,
                                    phone_number,
                                    name_prefix,
                                    first_name,
                                    middle_name,
                                    last_name,
                                    name_suffix,
                                    nickname,
                                ]
                db.close()

        # Build the array after dealing with all the files
        user_list = list(user_dictionary.values())

        if len(user_list) > 0:
            description = 'CloudKit Participants - CloudKit accounts participating in CloudKit shares.'
            report = ArtifactHtmlReport('Participants')
            report.start_artifact_report(report_folder, 'Participants', description)
            report.add_script()
            user_headers = (
                'Record ID',
                'Email Address',
                'Phone Number',
                'Name Prefix',
                'First Name',
                'Middle Name',
                'Last Name',
                'Name Suffix',
                'Nickname',
            )
            report.write_artifact_data_table(
                user_headers, user_list, '', write_location=False
            )
            report.end_artifact_report()

            tsvname = 'CloudKit Participants'
            tsv(report_folder, user_headers, user_list, tsvname)
        else:
            logfunc('No CloudKit - CloudKit Participants data available')



import glob
import os

import nska_deserialize as nd
from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class NotificationsXII(ab.Artifact):
    _name = 'iOS Notifications'
    _search_dirs = '*private/var/mobile/Library/UserNotifications*'
    _category = 'Notifications'

    def get(self, files_found, seeker):

        bundle_info = get_bundle_info(files_found)
        data_list = []
        exportedbplistcount = 0

        pathfound = str(files_found[0])
        # logfunc(f'Posix to string is: {pathfound}')
        for filepath in glob.iglob(pathfound + "/**", recursive=True):
            # create directory where script is running from
            if os.path.isfile(filepath):  # filter dirs
                file_name = os.path.splitext(os.path.basename(filepath))[0]
                # create directory
                if filepath.endswith('DeliveredNotifications.plist'):
                    bundle_id = os.path.basename(os.path.dirname(filepath))
                    # open the plist
                    p = open(filepath, "rb")
                    plist = nd.deserialize_plist(p)

                    # Empty plist will be { 'root': None }
                    if isinstance(plist, dict):
                        continue  # skip it, it's empty

                    # Good plist will be a list of dicts
                    for item in plist:
                        creation_date = ''
                        title = ''
                        subtitle = ''
                        message = ''
                        other_dict = {}
                        bundle_name = bundle_info.get(bundle_id, bundle_id)
                        # if bundle_name == 'com.apple.ScreenTimeNotifications':
                        #     pass # has embedded plist!
                        for k, v in item.items():
                            if k == 'AppNotificationCreationDate':
                                creation_date = str(v)
                            elif k == 'AppNotificationMessage':
                                message = v
                            elif k == 'AppNotificationTitle':
                                title = v
                            elif k == 'AppNotificationSubtitle':
                                subtitle = v
                            else:
                                if isinstance(v, bytes):
                                    logfunc(
                                        f'Found binary data, look into this one later k={k}!'
                                    )
                                elif isinstance(v, dict):
                                    pass  # recurse look for plists # TODO
                                elif isinstance(v, list):
                                    pass  # recurse look for plists # TODO
                                other_dict[k] = str(v)
                        if subtitle:
                            title += f'[{subtitle}]'
                        data_list.append(
                            (
                                creation_date,
                                bundle_name,
                                title,
                                message,
                                str(other_dict),
                            )
                        )
                    p.close()

                elif "AttachmentsList" in file_name:
                    pass  # future development

        description = 'iOS > 12 Notifications'
        report = ArtifactHtmlReport('iOS Notificatons')
        report.start_artifact_report(report_folder, 'iOS Notifications', description)
        report.add_script()
        data_headers = (
            'Creation Time',
            'Bundle',
            'Title[Subtitle]',
            'Message',
            'Other Details',
        )
        report.write_artifact_data_table(data_headers, data_list, filepath)
        report.end_artifact_report()

        logfunc("Total notifications processed:" + str(len(data_list)))
        # logfunc("Total exported bplists from notifications:" + str(exportedbplistcount))

        tsvname = 'Notifications'
        tsv(report_folder, data_headers, data_list, tsvname)

        tlactivity = 'Notifications'
        timeline(report_folder, tlactivity, data_list, data_headers)
        if len(data_list) == 0:
            logfunc("No notifications found.")


def get_bundle_id_and_names_from_plist(library_plist_file_path):
    """Parses Library.plist and returns a dictionary where Key=Bundle_ID, Value=Bundle_Name"""
    bundle_info = {}
    f = open(library_plist_file_path, 'rb')
    plist = nd.deserialize_plist(f)
    for k, v in plist.items():
        bundle_info[v] = k
    f.close()
    return bundle_info


def get_bundle_info(files_found):

    for file_path in files_found:
        file_path = str(file_path)
        if file_path.endswith('Library.plist') and os.path.dirname(file_path).endswith(
            'UserNotificationsServer'
        ):
            bundle_info = get_bundle_id_and_names_from_plist(file_path)
            return bundle_info
        # If this is fs search, then only top level folder will be present, so append path and search it too
        if file_path.endswith('UserNotificationsServer'):
            plist_file_path = os.path.join(file_path, 'Library.plist')
            if os.path.exists(plist_file_path):
                bundle_info = get_bundle_id_and_names_from_plist(plist_file_path)
                return bundle_info
    return {}



from dataclasses import dataclass

from xleapp import Artifact, WebIcon, Search, timed


@dataclass
class AggDictPasscodeType(Artifact):
    def __post_init__(self):
        self.name = "Aggregate Dictionary Passcode Type"
        self.category = "Aggregate Dictionary"
        self.web_icon = WebIcon.BOOK

    @timed
    @Search("*/AggregateDictionary/ADDataStore.sqlitedb")
    def process(self):
        for fp in self.found:
            cursor = fp().cursor()

            cursor.execute(
                """
                select
                date(dayssince1970*86400, 'unixepoch'),
                key,
                case
                when value=-1 then '6-digit'
                when value=0 then 'no passcode'
                when value=1 then '4-digit'
                when value=2 then 'custom alphanumeric'
                when value=3 then 'custom numeric'
                else "n/a"
                end "value"
                from
                scalars
                where key like 'com.apple.passcode.passcodetype%'
                """,
            )

            all_rows = cursor.fetchall()
            usageentries = len(all_rows)

            if usageentries > 0:
                data_list = []
                for row in all_rows:
                    data_list.append((row[0], row[1], row[2]))

                self.data = data_list



import glob
import os
import pathlib
import sqlite3

from html_report.artifact_report import ArtifactHtmlReport
from helpers import is_platform_windows, kmlgen, open_sqlite_db_readonly, timeline, tsv

from artifacts.Artifact import Artifact


class TileAppDB(ab.Artifact):
    _name = 'Tile App DB Info & Geolocation'
    _search_dirs = '*private/var/mobile/Containers/Shared/AppGroup/*/com.thetxleapp.tile-TileNetworkDB.sqlite*'
    _category = 'Locations'

    def get(self, files_found, seeker):
        for file_found in files_found:
            file_found = str(file_found)

            if file_found.endswith('tile-TileNetworkDB.sqlite'):
                break

        db = open_sqlite_db_readonly(file_found)
        cursor = db.cursor()
        cursor.execute(
            """
        SELECT
        datetime(ZTIMESTAMP,'unixepoch','31 years'),
        ZNAME,
        datetime(ZACTIVATION_TIMESTAMP,'unixepoch','31 years'),
        datetime(ZREGISTRATION_TIMESTAMP,'unixepoch','31 years'),
        ZALTITUDE,
        ZLATITUDE,
        ZLONGITUDE,
        ZID,
        ZNODE_TYPE,
        ZSTATUS,
        ZIS_LOST,
        datetime(ZLAST_LOST_TILE_COMMUNITY_CONNECTION,'unixepoch','31 years')
        FROM ZTILENTITY_NODE INNER JOIN ZTILENTITY_TILESTATE ON ZTILENTITY_NODE.ZTILE_STATE = ZTILENTITY_TILESTATE.Z_PK
        """
        )

        all_rows = cursor.fetchall()
        usageentries = len(all_rows)
        data_list = []
        if usageentries > 0:
            for row in all_rows:
                data_list.append(
                    (
                        row[0],
                        row[1],
                        row[2],
                        row[3],
                        row[4],
                        row[5],
                        row[6],
                        row[7],
                        row[8],
                        row[9],
                        row[10],
                        row[11],
                    )
                )

                description = ''
                report = ArtifactHtmlReport('Tile App - Tile Information & Geolocation')
                report.start_artifact_report(
                    report_folder, 'Tile App DB Info & Geolocation', description
                )
                report.add_script()
                data_headers = (
                    'Timestamp',
                    'Tile Name',
                    'Activation Timestamp',
                    'Registration Timestamp',
                    'Altitude',
                    'Latitude',
                    'Longitude',
                    'Tile ID',
                    'Tile Type',
                    'Status',
                    'Is Lost?',
                    'Last Community Connection',
                )
                report.write_artifact_data_table(data_headers, data_list, file_found)
                report.end_artifact_report()

                tsvname = 'Tile App DB Info Geolocation'
                tsv(report_folder, data_headers, data_list, tsvname)

                tlactivity = 'Tile App DB Info Geolocation'
                timeline(report_folder, tlactivity, data_list, data_headers)

                kmlactivity = 'Tile App DB Info Geolocation'
                kmlgen(report_folder, kmlactivity, data_list, data_headers)
        else:
            logfunc('No Tile App DB data available')

        db.close()



import os
import plistlib

from html_report.artifact_report import ArtifactHtmlReport
from helpers import tsv
from xleapp.html_report import Icon

from artifacts.Artifact import Artifact


class CellularWireless(ab.Artifact):

    _name = 'Cellular Wireless'
    _search_dirs = '*wireless/Library/Preferences/com.apple.*'
    _category = 'Cellular Wireless'
    _web_icon = WebIcon.BAR_CHART

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        for filepath in files_found:
            basename = os.path.basename(filepath)
            if (
                basename == "com.apple.commcenter.device_specific_nobackup.plist"
                or basename == "com.apple.commcenter.plist"
            ):
                p = open(filepath, "rb")
                plist = plistlib.load(p)
                for key, val in plist.items():
                    data_list.append((key, val, filepath))
                    if key == "ReportedPhoneNumber":
                        logdevinfo(f"Reported Phone Number: {val}")

                    if key == "CDMANetworkPhoneNumberICCID":
                        logdevinfo(f"CDMA Network Phone Number ICCID: {val}")

                    if key == "imei":
                        logdevinfo(f"IMEI: {val}")

                    if key == "LastKnownICCID":
                        logdevinfo(f"Last Known ICCID: {val}")

                    if key == "meid":
                        logdevinfo(f"MEID: {val}")

        location = 'see source field'
        report = ArtifactHtmlReport('Cellular Wireless')
        report.start_artifact_report(report_folder, 'Cellular Wireless')
        report.add_script()
        data_headers = ('Key', 'Values', 'Source')
        report.write_artifact_data_table(data_headers, data_list, location)
        report.end_artifact_report()

        tsvname = 'Cellular Wireless'
        tsv(report_folder, data_headers, data_list, tsvname)



import datetime
import json
import shutil
from os import listdir
from os.path import basename, dirname, isfile, join

from html_report.artifact_report import ArtifactHtmlReport
from helpers import timeline, tsv

from artifacts.Artifact import Artifact


class VoiceTriggers(ab.Artifact):
    _name = 'Voice Triggers'
    _search_dirs = ('**/td/audio/*.json', '**/td/audio/*.wav')
    _category = 'Voice-Triggers'

    def get(self, files_found, seeker):
        info_files = []
        data_list = []
        if len(files_found) > 1:
            for file_found in files_found:
                if file_found.endswith('wav'):
                    shutil.copyfile(
                        file_found, join(report_folder, basename(file_found))
                    )
                elif file_found.endswith('json') and 'meta_version.json' != basename(
                    file_found
                ):
                    info_files.append(file_found)

            info_files.sort()
            wav_files = [
                join(report_folder, file)
                for file in listdir(report_folder)
                if isfile(join(report_folder, file))
            ]
            wav_files.sort()

            for info_file, wav_file in zip(info_files, wav_files):
                if (
                    basename(info_file).split('.')[0]
                    == basename(wav_file).split('.')[0]
                ):
                    with open(info_file, "rb") as file:
                        fl = json.load(file)
                        if 'grainedDate' in fl:
                            creation_date = format_time(fl['grainedDate'])
                        else:
                            creation_date = ''

                        audio = """
                                <audio controls>
                                    <source src={} type="audio/wav">
                                    <p>Your browser does not support HTML5 audio elements.</p>
                                </audio>
                                """.format(
                            wav_file
                        )

                        data_list.append(
                            (
                                creation_date,
                                fl['productType'],
                                fl['utteranceWav'],
                                audio,
                            )
                        )

            report = ArtifactHtmlReport('Voice Triggers')
            report.start_artifact_report(report_folder, 'Voice Triggers')
            report.add_script()
            data_headers = ('Creation Date', 'Device', 'Path to File', 'Audio File')
            report.write_artifact_data_table(
                data_headers,
                data_list,
                dirname(file_found),
                html_no_escape=['Audio File'],
            )
            report.end_artifact_report()

            tsvname = 'Voice Triggers'
            tsv(report_folder, data_headers, data_list, tsvname)

            tlactivity = 'Voice Triggers'
            timeline(report_folder, tlactivity, data_list, data_headers)
        else:
            logfunc('No Voice Triggers found')


def format_time(date_time_str):
    date_time_obj = datetime.datetime.strptime(date_time_str, '%Y%m%d')
    formatted = '{}-{}-{}'.format(
        date_time_obj.year, date_time_obj.month, date_time_obj.day
    )
    return formatted



import json
import os

from helpers import tsv
from xleapp.html_report import Icon
from html_report.artifact_report import ArtifactHtmlReport

from artifacts.Artifact import Artifact


class DiscordManifest(ab.Artifact):

    _name = 'Discord Manifest'
    _search_dirs = '*/private/var/mobile/Containers/Data/Application/*/Documents/RCTAsyncLocalStorage_V1/manifest.json'
    _category = 'Discord'
    _web_icon = WebIcon.FILE_TEXT

    def __init__(self):
        super().__init__(self)

    def get(self, files_found, seeker):
        data_list = []
        for file_found in files_found:
            file_found = str(file_found)

            if os.path.isfile(file_found):
                with open(file_found) as f_in:
                    for jsondata in f_in:
                        jsonfinal = json.loads(jsondata)

            for key, value in jsonfinal.items():
                data_list.append((key, value))

        if len(data_list) > 0:
            report = ArtifactHtmlReport('Discord Manifest')
            report.start_artifact_report(report_folder, 'Discord Manifest')
            report.add_script()
            data_headers = ('Key', 'Value')
            report.write_artifact_data_table(data_headers, data_list, file_found)
            report.end_artifact_report()

            tsvname = 'Discord Manifest'
            tsv(report_folder, data_headers, data_list, tsvname)
